function [X] = normalize_X(X)
%Patch normalization to zero mean and unit l_2 norm.
%
%Argument:
%   X(:,t): t^th patch in vector form
%
%Output:
%   X: normalized image

%zero mean (x -> x - 1/n * sum_{i=1}^n} x_i):
    m = mean(X,1);
    X = X - repmat(m,[size(X,1) 1]);
    
%unit l_2 norm (x -> x/||x||_2):
    norm_X = sqrt(sum(X.^2,1));
    idxs = find(norm_X~=0);
    X = X(:,idxs);
    X = X ./ repmat(norm_X(idxs),[size(X,1) 1]);