function [X,m,norm_X,idxs] = normalize_masked_X(X,M)
%Patch normalization to zero mean and unit l_2 norm, using the available (determined by mask M) coordinates.
%
%Argument:
%   X(:,t): t^th patch in vector form
%   M: mask for the observable coordinates of X, "M(i,j)=1" = "X(i,j) is not observable", "M(i,j)=0" = "otherwise".
%
%Output:
%   X: normalized X.
%   m, norm_X, idx: describe the normalizing transformation.

T = size(X,2);

%zero mean:
    m = zeros(1,T);
    for t = 1 : T
        if sum(~M(:,t))==0%no measured quantities
            m(t) = 0;
        else
            m(t) = mean(X(~M(:,t),t));
        end
    end
    X = X - repmat(m,[size(X,1) 1]);
    
%unit l_2 norm:
    norm_X = zeros(1,T);
    for t = 1 : T
        if sum(~M(:,t))==0%no measured quantities
            norm_X(t) = 1;
        else
            norm_X(t) = sqrt(sum(X(~M(:,t),t).^2));
        end
    end
    idxs = find(norm_X~=0);
    X(:,idxs) = X(:,idxs) ./ repmat(norm_X(idxs),[size(X,1) 1]);
